from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QDoubleSpinBox, QSpinBox, QSlider

from pyqtschema.widgets.base import SchemaWidgetMixin, state_property


class SpinSchemaWidget(SchemaWidgetMixin, QSpinBox):

    @state_property
    def state(self) -> int:
        return self.value()

    @state.setter
    def state(self, state: int):
        self.setValue(state)

    def configure(self):
        self.valueChanged.connect(self.on_changed.emit)

        if "maximum" in self.schema:
            if "exclusiveMaximum" in self.schema:
                self.setMaximum(min(self.schema["maximum"], self.schema["exclusiveMaximum"] - 1))
            else:
                self.setMaximum(self.schema["maximum"])
        elif "exclusiveMaximum" in self.schema:
            self.setMaximum(self.schema["exclusiveMaximum"] - 1)
        if "minimum" in self.schema:
            if "exclusiveMinimum" in self.schema:
                self.setMinimum(min(self.schema["minimum"], self.schema["exclusiveMinimum"] + 1))
            else:
                self.setMinimum(self.schema["minimum"])
        elif "exclusiveMinimum" in self.schema:
            self.setMinimum(self.schema["exclusiveMinimum"] + 1)
        if "multipleOf" in self.schema:
            self.setSingleStep(self.schema["multipleOf"])


class SpinDoubleSchemaWidget(SchemaWidgetMixin, QDoubleSpinBox):

    @state_property
    def state(self) -> float:
        return self.value()

    @state.setter
    def state(self, state: float):
        self.setValue(state)

    def configure(self):
        self.valueChanged.connect(self.on_changed.emit)

        if "maximum" in self.schema:
            if "exclusiveMaximum" in self.schema:
                self.setMaximum(min(self.schema["maximum"], self.schema["exclusiveMaximum"]))
            else:
                self.setMaximum(self.schema["maximum"])
        elif "exclusiveMaximum" in self.schema:
            self.setMaximum(self.schema["exclusiveMaximum"])
        if "minimum" in self.schema:
            if "exclusiveMinimum" in self.schema:
                self.setMinimum(min(self.schema["minimum"], self.schema["exclusiveMinimum"]))
            else:
                self.setMinimum(self.schema["minimum"])
        elif "exclusiveMinimum" in self.schema:
            self.setMinimum(self.schema["exclusiveMinimum"])
        if "multipleOf" in self.schema:
            self.setSingleStep(self.schema["multipleOf"])


class IntegerRangeSchemaWidget(SchemaWidgetMixin, QSlider):

    def __init__(self, schema: dict, ui_schema: dict, widget_builder: 'WidgetBuilder', *args, **kwargs):
        kwargs['orientation'] = Qt.Horizontal
        super().__init__(schema, ui_schema, widget_builder, *args, **kwargs)

    @state_property
    def state(self) -> int:
        return self.value()

    @state.setter
    def state(self, state: int):
        self.setValue(state)

    def configure(self):
        self.valueChanged.connect(self.on_changed.emit)

        minimum = 0
        if "minimum" in self.schema:
            minimum = self.schema["minimum"]
            if self.schema.get("exclusiveMinimum"):
                minimum += 1

        maximum = 0
        if "maximum" in self.schema:
            maximum = self.schema["maximum"]
            if self.schema.get("exclusiveMaximum"):
                maximum -= 1

        if "multipleOf" in self.schema:
            self.setTickInterval(self.schema["multipleOf"])
            self.setSingleStep(self.schema["multipleOf"])
            self.setTickPosition(self.TicksBothSides)

        self.setRange(minimum, maximum)



from abc import abstractmethod


class IBuilder:
    """ Defines a builder-interface """

    @abstractmethod
    def create_widget(self, schema: dict, ui_schema: dict, state=None) -> 'SchemaWidgetMixin':
        raise NotImplementedError()



import sys
from pathlib import Path
from typing import Dict, Callable, Optional

from PyQt5.QtGui import QIcon
from PyQt5.QtWidgets import QApplication, QMainWindow, QMenu, QAction, QScrollArea, QStyle, QMessageBox

from .__version__ import version
from .builder import WidgetBuilder
from .schema import schema_from_json


def _exit(txt: str):
    if txt:
        print(txt)
    exit(1)


class MainWindow(QMainWindow):
    def __init__(self, schema: Dict, *args, **kwargs):
        super().__init__(*args, **kwargs)
        style = self.style()

        self.menu_file = QMenu('File', self.menuBar())
        self.menu_help = QMenu('Help', self.menuBar())

        load_action = self._create_action('Load data', triggered=self.load_file,
                                          icon=style.standardIcon(QStyle.SP_FileIcon))
        save_action = self._create_action('Save data', triggered=self.save_file,
                                          icon=style.standardIcon(QStyle.SP_DialogSaveButton))
        quit_action = self._create_action('Quit', triggered=self.close,
                                          icon=style.standardIcon(QStyle.SP_BrowserStop))
        for _act in (load_action, save_action, quit_action):
            self.menu_file.addAction(_act)

        about_action = self._create_action('About', triggered=self.about,
                                           icon=style.standardIcon(QStyle.SP_DialogHelpButton))
        self.menu_help.addAction(about_action)

        self.menuBar().addMenu(self.menu_file)
        self.menuBar().addMenu(self.menu_help)

        ui_schema = {}

        builder = WidgetBuilder(schema)
        form = builder.create_form(ui_schema)
        # form.widget.on_changed.connect(lambda d: print(dumps(d, indent=4)))

        widget = QScrollArea()
        widget.setWidget(form)
        widget.setWidgetResizable(True)
        self.setCentralWidget(widget)

    def _create_action(self, text: str, icon: Optional[QIcon] = None, tip: Optional[str] = None,
                       toggled: Optional[Callable] = None, triggered: Optional[Callable] = None, ) -> QAction:
        """Create a QAction"""
        action = QAction(text, self)
        if triggered is not None:
            action.triggered.connect(triggered)
        if toggled is not None:
            action.toggled.connect(toggled)
            action.setCheckable(True)
        if icon is not None:
            action.setIcon(icon)
        if tip is not None:
            action.setToolTip(tip)

        return action

    def load_file(self):
        pass

    def save_file(self):
        pass

    def about(self):
        _text = f'PyQtSchema: {version}'
        QMessageBox.about(self, "pyqtschema", _text)


def run(path_schema: Path, *qt_args):
    schema = schema_from_json(path)

    app = QApplication(*qt_args)

    main = MainWindow(schema)
    main.setWindowTitle(str(path_schema))
    main.show()

    app.exec()


if __name__ == '__main__':
    args = sys.argv[1:]

    if len(args) == 0:
        _exit('Specify a path to a json-schema')

    path = Path(args[0])

    _args = [] if len(args) == 1 else args[1:]

    run(path, _args)



import sys
from typing import Dict

from PyQt5.QtWidgets import QApplication, QScrollArea, QWidget

from pyqtschema.builder import WidgetBuilder


def build_example_widget(schema: Dict, ui_schema=None) -> QWidget:
    builder = WidgetBuilder(schema)
    form = builder.create_form(ui_schema=ui_schema)
    return form


def build_example(schema: Dict, scrollbar: bool = True):
    app = QApplication(sys.argv)
    widget = form = build_example_widget(schema)
    if scrollbar:
        widget = QScrollArea()
        widget.setWidget(form)
        widget.setWidgetResizable(True)

    widget.show()
    app.exec_()
